<?PHP
/*
  --------------------------------------------------------------------------
  Sphere Angular Diameter Calculator

  This program computes the apparent angular diameter of a sphere
  at any given distance from its center or above its surface.


  Author:
  Jay Tanner - 2019

  Current Web Site
  www.NeoProgrammics.com

  Development Language:
  PHP v7.1.8

  License:
  This program source code and any related data and graphics are released
  under the provisions of the Creative Commons Attribution-ShareAlike 4.0
  International.

  https://creativecommons.org/licenses/by-sa/4.0/legalcode
  --------------------------------------------------------------------------
*/


   ob_start();


   $cYear = date("Y");

   $_PROGRAM_VERSION_ = "3.5";

   $AuthorName = "Jay Tanner";

   $_ACTION_FILE_NAME_ = $_SERVER['SCRIPT_NAME'];

   $RanDist = 10*mt_rand(100, 999);

// Initialize error detection flag state
   $errorFlag = FALSE;

// Read radius and distance argument values
// from input interface.
   $R = trim(@$_POST['R']);
   $D = trim(@$_POST['D']);

// Set initial radius (of moon) and distance as default values.
   if ($R == "" && $D == "") {$R = 1079.35;   $D = 238856;}

//   $out = "";

// Read selected radio button value
   $radioButtonIndex = trim(@$_POST['DistOption']);

// Set default selected radio button to
// distance measured from center.
   if ($radioButtonIndex == ""){$radioButtonIndex = 0;}

// Set number of radio buttons.
// In this case 2 buttons.
   $n = 2;

// Create radio button states array
   for ($i=0;   $i < $n;   $i++)
       {
        if ($i == $radioButtonIndex)
           {$radioButton[$i] = "CHECKED";} else {$radioButton[$i] = "";}
       }

// Error if either argument is non-numeric
   if (!is_numeric($R) || !is_numeric($D))
      {
       $errorFlag = TRUE;
       $out = "<B style='color:red;'>ERROR: Both arguments should be positive numeric values.</B>\n";
      }

// Force both arguments to absolute values.
   if ($R < 0) {$R = abs($R);}
   if ($D < 0) {$D = abs($D);}

// Handle computation for distance
// measured from center of sphere.
   if ($radioButtonIndex == 0 && $errorFlag == FALSE)
{
   if ($R >= $D)
      {$errorFlag = TRUE;
       $out = "<B style='color:red;'><br>ERROR : &nbsp; The distance from the center must be greater than the radius.</B>\n<br><br>\n";
      }
   else
      {
       $AngDiamDeg = (acos(SqRt($D*$D - $R*$R) / $D) * 360 / M_PI);

       $AngDiamSS = sprintf("%1.3f", $AngDiamDeg * 3600);

   $AngDiamDMS = Deg_To_DMS ($AngDiamDeg, 3);
   $AngDiamDMS = str_replace("00&deg; ","", $AngDiamDMS);
   $AngDiamMM  = sprintf("%1.6f",  $AngDiamDeg*60);
   $AngDiamDeg = sprintf("%1.10f", $AngDiamDeg);

   $out = "<BR>Angular Diameter:<BR><BR><img src='images/Alpha-LC.png' style='border:none; box-shadow:none;'> = <code>$AngDiamDMS  =  $AngDiamMM&#39;  =  $AngDiamDeg&deg;</code>";
   $out .= "<BR><BR>The distance (<b>D</b>) to the eye is reckoned from the center of the sphere.<BR><BR>";

}
}


// Handle computation for distance
// measured from surface of sphere.
   if ($radioButtonIndex == 1 && $errorFlag == FALSE)
{
   if ($D <= 0 || $R < 0)
      {$errorFlag = TRUE;
       $out = "<B CLASS=\"TxRed\">Error : &nbsp; Radius and distance should both be greater than zero.</B>\n";
      }
   else
      {
      $AngDiamDeg = (aCos(SqRt(2*$R*$D + $D*$D) / ($R + $D)) * 360 / M_PI);

      $AngDiamDMS = Deg_To_DMS ($AngDiamDeg, 3);
      $AngDiamDMS = str_replace("00&deg; ","", $AngDiamDMS);
      $AngDiamMM  = sprintf("%1.6f",  $AngDiamDeg*60);
      $AngDiamDeg = sprintf("%1.10f", $AngDiamDeg);
      $AngDiamSS = sprintf("%1.3f", $AngDiamDeg * 3600);


   $out = "<br>Angular Diameter:<br><br><img src='images/Alpha-LC.png' style='border:none; box-shadow:none;'> = <code>$AngDiamDMS  =  $AngDiamMM&#39;  =  $AngDiamDeg&deg;</code>";
   $out .= "<br><br>The distance (<b>d</b>) to the eye is reckoned from the surface of the sphere.<br><br>";

      }
}

// Generate start of HTML page

print <<< _HTML

<!DOCTYPE HTML>

<html lang='en'>

<head>

<title>Apparent Angular Diameter of a Sphere at Any Distance</title>
<meta name='viewport' content='width=device-width, initial-scale=1'>
<meta http-equiv='content-type' content='text/html; charset=UTF-8'>
<meta name='description' content='This program computes the apparent angular diameter of a sphere at any given distance.'>
<meta name='keywords' content='angular diameter,angular diameter of sphere, sphere,PHPScienceLabs.com'>
<meta name='author' content='Jay Tanner - PHPScienceLabs.com'>
<meta http-equiv='pragma'  content='no-cache'>
<meta http-equiv='expires' content='-1'>
<meta name='robots'    content='index,follow'>
<meta name='googlebot' content='index,follow'>
<link rel="canonical" href="https://www.phpsciencelabs.com/sphere-angular-diameter-vs-distance-calculator/">


<style type='text/css'>
BODY {background-color:black; color:black; font-family:Verdana,sans-serif; font-size:100%;}

A:hover, A:active {background-color:transparent; color:red; font-family:Verdana,sans-serif; font-size:85%; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

A:link, A:visited {background-color:transparent; color:blue; font-family:Verdana,sans-serif; font-size:85%; font-weight:bold; text-decoration:none; line-height:150%; padding:2px;}

CODE{color:black; font-family:monospace; font-size:16px; font-weight:bold;}

DIV {background-color:white; color:black; font-size:100%; text-align:justify; padding:4px;}


PRE {font-family:monospace; font-size:15px; font-weight:bold; padding:4px; border:1px solid black;}

TABLE {background-color:black; font-family:Verdana; font-size:15px;}

TD {background-color:white; color:black; font-family:Verdana; font-size:15px; text-align:center;}

HR {background-color:black; height:2px; border:0px;}

IMG {border:1px solid #BBBBBB; box-shadow:6px 6px 6px #999999; border-radius:8px;}

INPUT[type="radio"]:checked {box-shadow:3px 3px 4px #999999; background-color:white; border:1px solid black;}

INPUT[type="submit"] {box-shadow:3px 3px 4px #999999;}

INPUT[type="text"]::-ms-clear {width:0; height:0;}
INPUT[type="text"] {box-shadow:3px 3px 4px #999999; background-color:white; font-family:monospace; font-size:20px; font-weight:bold; text-align:center; border:2px solid black; border-radius:4px;}
INPUT[type="text"]:focus {box-shadow:3px 3px 4px #999999;background-color:white; font-family:monospace; font-size:20px; border:2px solid red; text-align:center; font-weight:bold; border-radius:4px;}

TEXTAREA {box-shadow:3px 3px 4px #999999; background-color:LightYellow; color:black; font-family:monospace; font-size:20px; font-weight:bold; border:1px solid black; padding:4px;}

::-ms-selection  {background-color:yellow; color:black;}  /* Only applies to IE9+ and Safari     */
::-moz-selection {background-color:yellow; color:black;}  /* Only applies to Mozilla and FireFox */
</style>


</head>

<body bgcolor="white">

<table align='left' width="650" cellpadding="4" cellspacing="1">
<tr><td style='background-color:#000066; color:white; font-weight:normal; border:2px solid white; padding:4px; font-size:120%; border-radius:8px 8px 0px 0px;'>Apparent Angular Diameter of a Sphere at Any Distance<br><span style='font-size:80%;'>PHP Program by Jay Tanner</td></tr>

<tr><td>

<DIV style='font-family:Verdana; font-size:90%;'>

This program computes the apparent angular diameter of a simple sphere at any distance from the eye as reckoned from the center or surface of the sphere.

<br><br>
The units are dimensionless, which means that <code>$RanDist</code> units can mean whatever you decide, such as <code>$RanDist</code> miles, kilometers, feet, meters, etc., just as long as both the radius and distance use the same units.&nbsp;
<br><br>
The default initial values are the mean radius of the moon and its mean geocentric distance from Earth, both in statute miles.

<br><br>
<form name="form1" action="$_ACTION_FILE_NAME_" method="post">

<table width="650" cellpadding="6" cellspacing="1" align="center">

<TR><TD style='color:black; background-color:white;' ALIGN="center"><B>Radius</B></TD><TD style='color:black; background-color:LightCyan;'><INPUT NAME="R" TYPE="text" SIZE="24" VALUE="$R" MAXLENGTH="24"> units</TD></TR>

<TR><TD style='color:white; background-color:black;' ALIGN="center"><B>Distance</B></TD><TD style='color:black; background-color:LightCyan;'><INPUT NAME="D" TYPE="text" SIZE="24" VALUE="$D" MAXLENGTH="24"> units</TD></TR>

<TR><TD style='color:black; background-color:white;' ALIGN="center" COLSPAN="2"><B style='font-size:80%;'>Distance to the Eye Reckoned From:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</B>Center&nbsp;<INPUT NAME="DistOption" TYPE="radio" VALUE="0" $radioButton[0]>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Surface&nbsp;<INPUT NAME="DistOption" TYPE="radio" VALUE="1" $radioButton[1]></TD></TR>

<TR><TD COLSPAN="2" style='color:black; background-color:#DDDDDD;' ALIGN="center"><INPUT NAME="Compute_Button" TYPE="submit" VALUE="Compute"></TD></TR>

<TR><TD COLSPAN="2" style='color:black; background-color:white;' ALIGN="center">$out</TD></TR>

</TABLE>

</FORM>




<br>
<div style='font-size:100%; line-height:150%;'>
The simple formulas defined below take into account the actual forward curvature of the spherical surface towards the eye and 3D perspective with respect to the eye point rather than treating the sphere as a flat circular disk of the same radius as viewed from a distance.&nbsp; This is extremely important when viewing a sphere from relatively not too far away, such as a planet or the lunar surface as viewed from a spacecraft only a few thousands of kilometers or miles away rather than hundreds of thousands or more.
<br><br>
Only for great distances from the eye, can we treat a sphere as a flat circular disk, but for much closer distances, we need to account for the perspective to obtain the correct angular diameter.&nbsp; The equations for this are actually quite simple and come from elementary trigonometry translated in a physical context.
</div>

<br><br>

<TABLE width="650" cellpadding='8' style='border:none;'>
<TR>
<TD style='border:none;'><IMG SRC="images/sphere_ang_diam_from_center.png" ALT=""></TD>
<TD VALIGN="middle">
<div style='font-size:13px; text-align:center;'><B>Where the Distance to the Eye is Reckoned<br>From the Center of the Sphere:</B><DIV>
<BR>
<div style='font-size:13px;'>
Given the radius (<b>R</b>) of the sphere and the distance (<b>D</b>) to the eye point, as measured from the center of the sphere, the angular diameter (<img src='images/Alpha-LC.png' style='border:none; box-shadow:none;'>) of the sphere as viewed from that point is:
<BR><BR>
<img src='images/eq1.png' style='border:1px solid #BBBBBB;'>

</DIV>
</TD>
</TR>
</TABLE>

<BR>

<TABLE width="650" cellpadding='8' style='border:none;'>
<TR>
<TD><IMG SRC="images/sphere_ang_diam_from_surface.png" ALT=""></TD>
<TD VALIGN="middle">
<div style='font-size:13px; text-align:center;'><B>Where the Distance to the Eye is Reckoned<br>From the Surface of the Sphere:</B><DIV>
<BR>
<div style='font-size:13px;'>
Given the radius (<b>R</b>) of the sphere and the distance (<b>d</b>) to the eye point, as measured from the surface, the general equation for the angular diameter (<img src='images/Alpha-LC.png' style='border:none; box-shadow:none;'>) of the sphere as viewed from that point is:

<BR><BR>
<img src='images/eq2.png' style='border:1px solid #BBBBBB;'>

</DIV>
</TD>
</TR>


</TABLE>

</DIV>

<DIV style='font-size:60%; text-align:center;'><B>$AuthorName - $cYear</B></DIV>

</TD></TR></TABLE>

<br><br><br><br><br><br><br><br><br><br><br><br><br><br>
<br><br><br><br><br><br><br><br><br><br><br><br><br><br>

</BODY>
</HTML>

_HTML;





/*
   ========================================================================
   Convert decimal degrees to degrees, minutes, seconds of arc.
   The angular elements are returned in a space-delimited string.

   No symbols are attached to the angle elements. The seconds
   can optionally be rounded to up to 3 decimals with 0=Default.

   ========================================================================
*/

   function Deg_To_DMS ($AngleDeg, $ssDecimals=0, $PosSignFlag=FALSE, $SymbolsFlag=TRUE)
{

// Init
   $_d_ = $_m_ = $_s_ = '';

// ------------------------------------------------------
// Compute signed angular elements from decimal argument.
// This value is taken as decimal degrees.

   $sign = ($AngleDeg < 0)? '-' : '';

   if ($PosSignFlag === TRUE and $sign == '') {$sign = '+';}

   $degrees = abs($AngleDeg);
   $dd      = floor($degrees);
   $minutes = 60*($degrees - $dd);
   $mm      = floor($minutes);
   $seconds = 60*($minutes - $mm);
   $ss      = sprintf("%1.3f", $seconds);

// ----------------------------------
// Check for that blasted 60s glitch.

   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $dd++;}

// ----------------------------
// Format the angular elements.

   $dd = sprintf("%02d",  $dd);
   $mm = sprintf("%02d",  $mm);
   $ss = sprintf("%1.$ssDecimals" . "f", $ss);

   if ($ss < 10) {$ss = "0$ss";}

// Attach optional DMS symbols.
// Default = TRUE.
   if ($SymbolsFlag === TRUE)
      {
       $_d_ = "&deg;";
       $_m_ = "'";
       $_s_ = '"';
      }

// Done.
   return "$sign$dd$_d_ $mm$_m_ $ss$_s_";

} // End of  Deg_To_DMS(...)






?>


